;This is a quick sample script showing how to use some of the features of rv_fit_mp
;The heart of the program is rvlin.pro, which is the 'user-supplied function' expected by mpfit.pro.  RVLIN could also be used in something like an MCMC code for GLOBAL fitting of a system with many planets or velocities where a "by hand" or "brute force" search is not desireable.
;
;The linearization scheme and explicit (analytic) derivative magic of RVLIN is described in:
;
;Wright & Howard 2009 ApJS,182,205
;
;
;
;
; PLEASE REPORT ALL BUGS TO: Jason Wright <astrowright@gmail.com>
;
;
;
; RV_FIT_MP is a local fitter.  You'll have to search for planets 'by hand', but if your parameters are close, it will close in on the local chi^2 minimum very efficiently.  You need only supply the period closely in practice, but may also supply starting guesses for eccentricity and periastron passage.   Other parametrs are fit exactly, and cannot be given starting guesses.  
;
;The only case where you can fit "Blind" with no guesses, in the case of a single strong planetary signal.
;In this case, the code will run a periodogram on the data and use the tallest peak as its starting guess.
;
;
;To use this sample script, copy the uncommented lines into your idl window, or else just type .run readme
;
;It will assume that it being called from a directory that contains 12661.vel.txt (you can use IDL's 'cd' procedure to change your directory) and that all of the programs in the exoplanets.org/code/rvlin directory are in your IDL path.
;
;
;You will need the Goddard IDL astronomy user's library, and MPFIT.PRO from Craig Marqwardt:
;http://idlastro.gsfc.nasa.gov/
;http://cow.physics.wisc.edu/~craigm/idl/fitting.html
;
;
;We're going to use the custom plotting symbol ps=8, so you can use this line to define circles, or leave it out if you have your own
;
junk = circ(/fill)

;First, we use the readcol routine to read in the sample data file.  READCOL is available from the Goddard IDL library, which you will need to run these routines, and has other dependencies from the Goddard library.

readcol, '12661.vel.txt', t, v, e, tel, format = '(d,d,d,a)'  ;Read the sample data file
;
; Take a look at the structure of this file.  The times are unsorted, and the telescope indicator has three string values: K1, K2, and L.  These represent pre-upgrade Keck/HIRES, post-upgrade Keck/HIRES, and Lick velocities.  Note that the times are unsorted and the telescope strings are arbitrary.  RVLIN will automatically sort the times, and group all (t,v,e) triplets with the same telescope string.  The number of unique strings is the number of telescopes, and RVLIN will solve for the arbitrary offsets between them.
;
;
;Next we'll just try to fit blind.  This can only be done when there is a strong signal.
;
;
bestpars = rv_fit_mp(t, v, e, telvec = tel, offset = offset, /plot) ;first planet
;
; The telvec parameter is telling rv_fit_mp which telescope provided which velocities.  
;
; The code has decided to phase things up so the plot doesn't get too crowded, but for longer period planets it will show the entire time series.  RVLIN does a good job of finding the first, eccentric planet in this system.  The offset parameter contains the offsets between K1, K2, and L:
print, offset
;      -1.7920237       9.9422186
;
; Indicating that the Keck upgrade produced a 1.8 m/s offset in the HD 12661 data stream (it is different for every star, but usually < 5 m/s) and that Keck and Lick have a 9.9 m/s difference in their arbitrary zero point (this number can be almost arbitrarily large between unrelated telescopes).
;
;
; Now, let's look at the residuals.
;
vres = v-rv_drive(t, bestpars)
;
; RV_DRIVE takes the output of rv_fit_mp and produces a synthetic RV curve
;
; OK, let's ask rv_fit_mp to fit the residuals blind to find a second planet.  
;
bestpars2 = rv_fit_mp(t, vres, e, telvec = tel, /plot)
;
;Now that we have a good fit for the outer planet, we can go for a joint, 2-planet fit
;We use the orbel keyword to provide the starting guess -- a concatination of the 1- and 2-planet fits
;
bestparsfinal = rv_fit_mp(t, v, e, telvec = tel, orbel = [bestpars, bestpars2], tveout = tve)
;
;tveout returns a structure containing the times, velocities, and errors of the joint fit WITH THE OFFSET BETWEEN THE TELESCOPES APPLIED.
;
;
; This allows us to plot up the joint telescope, 2-planet fit along with the best-fit RV curve:
;
len = (max(t)-min(t))
m = len*1.2*findgen(1000)/999.+min(t)-len*0.1
plot, jdcal(tve.jd),  tve.mnvel, ps = 8
oplot,  jdcal(m), rv_drive(m, bestparsfinal)
;
;Ta-da.
;
;
;
;
;Now, let's look at two more examples:  how to fix parameters, and how to fix transit centers.
;
;
;Let's grab the RV curve for HD 209458 (Hi, Dave!):
;

readcol, '209458.vel.txt', t209, v209, e209, tel209, format = '(d,d,d,a)'  ;Read the sample data file
;
;This one's easy, so let's do a blind fit.
;
pars209 = rv_fit_mp(t209, v209, e209, telvec = tel209, /plot)
;
;The fit's not perfect because of the R-M effect near transit.  Note that the fitter got "stuck" at e=0.  This is a natural consequence of the LM fitter in the phase space of orbital parameter space as we are defining it, not a bug per se in RVLIN.  But it can be annoying.  I usually "jiggle" the starting parameters by small values and do many fits to and look for the one that "shook" the fitter out of its rut.

;OK, let's find the R-M effect and remove it.  Let's ask newpars.pro (available in the boottran package in the same place as RVLIN) for the transit times.  
;
;result = newpars(Per, ecc, omega, t_peri)
;
;where result contains [Per, ecc*cos(omega), ecc*sin(omega), t_transit]
;
;we get these from the pars209 vector, which contains [Per, T_peri, ecc, omega, K, gamma, dv/dt]
;
peot =  newpars(pars209[0], pars209[2], pars209[3], pars209[1])
;
per = peot[0]
tt = peot[3]
;
;Let's grab anything within 0.05 of transit center and toss it:
;
phase = (t209-tt)/per + 0.5  ;phase
phase = phase - floor(phase)  ;subtract integer number of orbits
plot,  phase,  v209, ps = 8

bad =  where(phase lt 0.55 and phase gt 0.45)
remove, bad, t209, v209, e209, tel209, phase
;
; And refit:
;
pars209 = rv_fit_mp(t209, v209, e209, telvec = tel209, /plot)
;
; Now, the fitter finds a non-zero eccentricity.  That's wrong (physically, not algorithmically), so let's force a circular fit: 
;
pars209 = rv_fit_mp(t209, v209, e209, telvec = tel209, /plot, /circ)
;
; Take a look at the free and fixed parameters:  t_peri is now fixed because the phase of the planet in the case of a circular orbit it is degenerate with omega, which must float in RVLIN.  If I had specified a nonzero ecc, tp would still be floating
;
;
; We could have also asked for a circular fit for one planet in a multiplanet fit with, e.g. circ = [0,1,0], or, in the case of HD 209458, circ = [1]
;
;
; We can actually fix ANY of the parameters Per, ecc, or t_p "by hand" with a vector of booleans.  For instance, to fix only the eccentricity at zero, do this:
;
pars209 = rv_fit_mp(t209, v209, e209, telvec = tel209, /plot, eccs = 0, fixed = [0, 0, 1, 0, 0, 0, 1])
;
;Note several curiosities here:
;1) I have included all 7 orbital elements in the fixed vector, because I wanted to illustrate the form, but I could have only specified the first 3 and the others would go to their default values.
;
;2) Here, I have asked RVLIN to fix ECC at the value I specified with "eccs = 0" - if I had not specified ecc it would have been fixed at 0.3, the default value.
;                                                                                   
;3) Only four parameters can actually be fixed:  Per, Tp, ecc, and dv/dt.  If you set the final fixed parameter to zero, it is equivalent to throwing the /trend flag (dv/dt can float).  Setting the fixed flag of the other 3 parameters (omega, K, gamma) to 1 has no effect.
;
; OK, final niftiness:  HD 209458 has a known phase at a given epoch because it transits.  Let's tell RVLIN when it actually transits and ask it to fit the RV curve subject to that constraint:
;
;
tt_known =  2452826.628514d ;From exoplanets.org #shamelessplug

pars209_transit =  rv_fit_mp(t209, v209, e209, telvec = tel209, /plot, /circ, tt = tt_known)

;Take a look at the output now.  Omega is forced to be 90 degrees (an arbitrary convention) so that t_peri = t_transit, but if ecc > 0 then it would be "tied" to the values of t_peri and t_transit, as appropriate.
;
;
;Finally, the period is often also known through transit work.  If the orbit is circular, all RVLIN is really doing is is solving for K, and gamma the amplitude and offset of a sinusoid with known phase and period.  This is using a sledgehammer to push in a tack (it's a linear problem, so we don't even need to invoke an L-M fitter), but that's OK, RVLIN won't complain and will just solve the matrix for you:
;
p_known = 3.52474859d
pars209_transit2 =  rv_fit_mp(t209, v209, e209, telvec = tel209, /plot, tt = tt_known, pers = p_known, eccs = 0, fixed = [1, 0, 1])

; You can specify multiple known fixed transit times (for the Kepler multis) by sending in a vector to tt with length equal to the number of transiting planets.  If there are also non-transiting planets in the system, they will need to be after the transiting planets in the list of parameters (that is, you can't specify tt = [t1, NaN, t2] in a 3 planet system where 2 transit, you have to send in tt = [t1,t2], even if there are 3 planets.  In this case, orbel would have to be a 7*3 = 21 element array, the first 14 of which describe the transiting planets, or pers a 3-element array with the first two elements the periods of the transiting planets, etc.).


;Have fun!
end
